"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _findUp = require("find-up");

var _findUp2 = _interopRequireDefault(_findUp);

var _loadJsonFile = require("load-json-file");

var _loadJsonFile2 = _interopRequireDefault(_loadJsonFile);

var _semver = require("semver");

var _semver2 = _interopRequireDefault(_semver);

var _PackageUtilities = require("./PackageUtilities");

var _PackageUtilities2 = _interopRequireDefault(_PackageUtilities);

var _Package = require("./Package");

var _Package2 = _interopRequireDefault(_Package);

var _NpmUtilities = require("./NpmUtilities");

var _NpmUtilities2 = _interopRequireDefault(_NpmUtilities);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var DEFAULT_PACKAGE_GLOB = "packages/*";

var Repository = function () {
  function Repository() {
    _classCallCheck(this, Repository);

    // findUp returns null when not found, and path.resolve starts from process.cwd()
    var lernaJsonLocation = _findUp2.default.sync("lerna.json") || _path2.default.resolve("lerna.json");

    this.rootPath = _path2.default.dirname(lernaJsonLocation);
    this.lernaJsonLocation = lernaJsonLocation;
    this.packageJsonLocation = _path2.default.join(this.rootPath, "package.json");
  }

  _createClass(Repository, [{
    key: "isCompatibleLerna",
    value: function isCompatibleLerna(cliVersion) {
      return _semver2.default.satisfies(cliVersion, "^" + this.initVersion);
    }
  }, {
    key: "isIndependent",
    value: function isIndependent() {
      return this.version === "independent";
    }
  }, {
    key: "buildPackageGraph",
    value: function buildPackageGraph() {
      this._packages = _PackageUtilities2.default.getPackages(this);
      this._packageGraph = _PackageUtilities2.default.getPackageGraph(this._packages);
    }
  }, {
    key: "hasDependencyInstalled",
    value: function hasDependencyInstalled(dependency, version) {
      return _NpmUtilities2.default.dependencyIsSatisfied(this.nodeModulesLocation, dependency, version);
    }
  }, {
    key: "lernaJson",
    get: function get() {
      if (!this._lernaJson) {
        try {
          this._lernaJson = _loadJsonFile2.default.sync(this.lernaJsonLocation);
        } catch (ex) {
          // No need to distinguish between missing and empty,
          // saves a lot of noisy guards elsewhere
          this._lernaJson = {};
        }
      }

      return this._lernaJson;
    }
  }, {
    key: "initVersion",
    get: function get() {
      return this.lernaJson.lerna;
    }
  }, {
    key: "version",
    get: function get() {
      return this.lernaJson.version;
    }
  }, {
    key: "nodeModulesLocation",
    get: function get() {
      return _path2.default.join(this.rootPath, "node_modules");
    }
  }, {
    key: "packageConfigs",
    get: function get() {
      return this.lernaJson.packages || [DEFAULT_PACKAGE_GLOB];
    }
  }, {
    key: "packages",
    get: function get() {
      if (!this._packages) {
        this.buildPackageGraph();
      }

      return this._packages;
    }
  }, {
    key: "packageGraph",
    get: function get() {
      if (!this._packageGraph) {
        this.buildPackageGraph();
      }

      return this._packageGraph;
    }
  }, {
    key: "packageJson",
    get: function get() {
      if (!this._packageJson) {
        try {
          this._packageJson = _loadJsonFile2.default.sync(this.packageJsonLocation);
        } catch (ex) {
          // try again next time
          this._packageJson = null;
        }
      }

      return this._packageJson;
    }
  }, {
    key: "package",
    get: function get() {
      if (!this._package) {
        this._package = new _Package2.default(this.packageJson, this.rootPath);
      }

      return this._package;
    }

    // Legacy

  }, {
    key: "versionLocation",
    get: function get() {
      return _path2.default.join(this.rootPath, "VERSION");
    }
  }]);

  return Repository;
}();

exports.default = Repository;
module.exports = exports["default"];